import java.io.*;
import java.util.*;
import java.util.regex.*;
import java.nio.channels.FileChannel;

public class EPubCompiler{
	//t@Cmymetype@ŏ̃GgƂĈk邱
	boolean cancelPressed = false;
	PankuzuCompilerPanel zipCompiler = new PankuzuCompilerPanel();
	public void cancel(){
		cancelPressed = true;
		zipCompiler.cancel();
	}
	public static File makeRandomDir(){
		String fname = "__tmp__";
		File ret = new File(fname);
		while(ret.exists()){
			ret = new File("__tmp__"+String.valueOf(Math.random()).replaceAll("[^0-9]",""));
		}
		ret.mkdir();
		return ret;
	}
	public static String getRandomName(File parentdir,String head,String tail){
		
		String fname = head+"_0"+tail;
		File ret = new File(parentdir.getPath()+"/"+fname);
		int count = 1;
		while(ret.exists()){
			fname =  head+"_"+String.valueOf(count++)+(String.valueOf(Math.random())+"0000").replaceAll("[^0-9]","").substring(0,3)+tail;
			ret = new File(parentdir.getPath()+"/"+fname);
		}
		return fname;
		
	}
	
	public static String getExtensionCode(String name){
		Pattern pat = Pattern.compile("(\\.[^.]+)$");
		Matcher mat = pat.matcher(name);
		if(mat.find()){
			return mat.group(1);
		}
		return "";
	}
	
	public void setProgressLabel(javax.swing.JLabel jl){
		zipCompiler.setProgressLabel(jl);
	}
	public void compile(String filename,EPubContent ec){
		File dir = makeRandomDir();
		ArrayList<File> al = new ArrayList<File>();
		ArrayList<String> entries = new ArrayList<String>();
		ArrayList<File> files = new ArrayList<File>();
		try{
			File mime = new File(dir.getPath()+"/mimetype");
			FileWriter fw = new FileWriter(mime);
			BufferedWriter bw = new BufferedWriter(fw);
			bw.write(ec.getMimeType());
			bw.close();
			
			String ename = mime.getPath().replaceAll("\\\\","/").replace(dir.getPath().replaceAll("\\\\","/")+"/","");
			
			files.add(mime);
			entries.add(ename);
		}catch(Exception e){
			e.printStackTrace();
		}
		
		try{
			File metadir = new File(dir.getPath()+"/META-INF");
			metadir.mkdir();
			FileWriter fw = new FileWriter(new File(metadir.getPath()+"/container.xml"));
			BufferedWriter bw = new BufferedWriter(fw);
			bw.write(ec.getContainerString());
			bw.close();
			al.addAll(getAllFiles(metadir));
		}catch(Exception e){
			e.printStackTrace();
		}
		try{
			
			File oebpsdir = new File(dir.getPath()+"/OEBPS");
			oebpsdir.mkdir();
			ec.makeOPFFile(oebpsdir);
			ec.makeNCXFile(oebpsdir);
			al.addAll(getAllFiles(oebpsdir));
		}catch(Exception e){
			e.printStackTrace();
		}
		for(int ii = 0;ii < al.size();ii++){
			if(cancelPressed){
				return;
			}
			File f = al.get(ii);
			if(f.isFile()){
				String ename = f.getPath().replaceAll("\\\\","/").replace(dir.getPath().replaceAll("\\\\","/")+"/","");
				
				files.add(f);
				entries.add(ename);
			}
		}
		
		zipCompiler.zip(filename,files,entries);
		
		PankuzuCompilerPanel.deleteFiles(dir);
		
	}
	public static ArrayList<File> getAllFiles(File f){
		ArrayList<File> ret = new ArrayList<File>();
		if(f.exists()){
			if(f.isFile()){
				ret.add(f);
			}else if (f.isDirectory()){
				//System.out.println(f.getName());
				File[] flist = f.listFiles();
				int num = 0;
				for(int ii = 0;ii < flist.length;ii++){
					if(!f.getName().equals("..") && !f.getName().equals(".")){
						ret.addAll(getAllFiles(flist[ii]));
						num++;
					}
				}
				if(num == 0){
					ret.add(f);
				}
			}
			
		}else{
			System.out.println(f.getPath()+" does not exist.");
		}
		return ret;
	}
	public static void main(String[] args){
		EPubCompiler comp = new EPubCompiler();
		EPubContent ep = new EPubContent();
		
		XHTMLComicPage cp1 = new XHTMLComicPage("contents1.xhtml",new File("test1.jpg"));
		XHTMLComicPage cp2 = new XHTMLComicPage("contents2.xhtml",new File("test2.jpg"));
		
		ep.addContentsPage(cp1);
		ep.addContentsPage(cp2);
		AuthorData ad = new AuthorData();
		ad.setAuthorName("tester");
		ad.setAuthorWebsiteName("tester");
		ad.setAuthorWebsite("http://www.yahoo.co.jp");
		ep.setAuthorData(ad);
		ep.addAuthorInfoPage();
		comp.compile("test__.epub",ep);
	}
	
}



class EPubContent{
	String mimeType = "application/epub+zip";
	String author = "anonymous";
	String authorWebsiteAddress = "";
	String authorWebsiteName = "";
	String publisherWebsiteAddress = "";
	
	boolean rtlFlag = false;
	String title ="nullpo";
	String language = "ja";
	String publisher = "nullpo";
	String license = "undef";
	String containerString = "";
	String cssString = "";
	String uid = "epub"+String.valueOf(Math.random()).replaceAll("[^0-9]","");
	ArrayList<EPubPage> pages = new ArrayList<EPubPage>();
	ArrayList<EPubIndex> indices = new ArrayList<EPubIndex>();
	String ncxFileName = "toc.ncx";
	String opfFileName = "content.opf";
	EPubContent(){
		containerString = getDefaultContainerXMLString();
	}
	public void setRtLFlag(boolean b){
		rtlFlag = b;
	}
	public void setMimeType(String s){
		mimeType = s;
	}
	public void setContainerString(String s){
		containerString = s;
	}
	public void setTitle(String t){
		title = t;
	}
	public void setAuthorData(AuthorData au){
		author = au.getAuthorName();
		publisher = au.getPublisherName();
		
		authorWebsiteAddress = au.getAuthorWebsite();
		authorWebsiteName = au.getAuthorWebsiteName();
		publisherWebsiteAddress = au.getPublisherWebsite();
		
	}
	public String getMimeType(){
		return mimeType;
	}
	public String getContainerString(){
		return containerString;
	}
	public void addBlankPage(){
		EPubPage ep = new EPubPage("blank");
		ep.setContentString("");
		addContentsPage(ep);
	
	}
	public void addAuthorInfoPage(){
		EPubPage ep = new EPubPage("authorinfo");
		StringBuffer sb = new StringBuffer();
		sb.append("<div style=\"text-align:center;\">");
		
		sb.append("<br/><br/><br/><strong>"+author+"</strong><br/>");
		if(authorWebsiteAddress.length() > 0){
			sb.append("<a href=\""+authorWebsiteAddress+"\" style=\"color:black;text-decoration:none;\">"+authorWebsiteName+"</a><br/><br/>");
		}
		sb.append("<b>"+publisher+"</b><br/>");
		if(publisherWebsiteAddress.length() > 0){
		
			sb.append("<small>"+"<a href=\""+publisherWebsiteAddress+"\" style=\"color:black;text-decoration:none;\">"+publisherWebsiteAddress+"</a></small>");
		}
		sb.append("</div>");
		sb.append("<div style=\"text-align:center;\">");
		sb.append("<br/><br/>");
		sb.append("Compiled by <a href=\"http://sesamecake.sakura.ne.jp/picopounder/\" style=\"color:black;text-decoration:none;\">PiCoPounder</a>.<br/><br/>");
		sb.append("</div>");
		
		ep.setContentString(sb.toString());
		addContentsPage(ep);
	}
	public void addContentsPage(EPubPage ep){
		pages.add(ep);
	}
	public void addIndex(EPubIndex ei){
		indices.add(ei);
	}
	public void addIndex(EPubPage p ,String desc){
		addIndex(new EPubIndex(p,desc));
		
	}
	public void addIndex(EPubPage p ,String desc,int pnum){
		addIndex(new EPubIndex(p,desc,pnum));
		
	}
	public static String getDefaultContainerXMLString(){
		StringBuffer sb = new StringBuffer();
		sb.append("<?xml version=\"1.0\"?><container version=\"1.0\" xmlns=\"urn:oasis:names:tc:opendocument:xmlns:container\">\n");
		sb.append("<rootfiles>\n");
    	sb.append("<rootfile full-path=\"OEBPS/content.opf\" media-type=\"application/oebps-package+xml\"/>\n");
  		sb.append("</rootfiles>\n");
		sb.append("</container>\n");
		return sb.toString();
	}
	public void makeOPFFile(File dir) throws IOException{
		
		BufferedWriter cssr = new BufferedWriter(new FileWriter(new File(dir+"/style.css")));
		cssr.write(cssString);
		cssr.close();
		for(int ii = 0;ii < pages.size();ii++){//----create contents
			pages.get(ii).setContentDirPath(dir.getPath());
			pages.get(ii).setStyleSheet("style.css");
			pages.get(ii).createPageFile(dir.getPath()+"/contents"+String.valueOf(ii)+".xhtml","resource");
		}
		OutputStreamWriter fw = new OutputStreamWriter(new FileOutputStream(new File(dir.getPath()+"/"+opfFileName)),"UTF-8");
		BufferedWriter br = new BufferedWriter(fw);
		br.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
		br.write("<package version=\"2.0\" xmlns=\"http://www.idpf.org/2007/opf\" unique-identifier=\"BookId\">\n");
		br.write("<metadata xmlns:dc=\"http://purl.org/dc/elements/1.1/\" xmlns:opf=\"http://www.idpf.org/2007/opf\">\n");
		br.write("<dc:title>\n"+title+"</dc:title> ");
		br.write("<dc:creator opf:role=\"aut\">\n"+author+"</dc:creator>\n");
		br.write("<dc:language>\n"+language+"</dc:language> ");
		br.write("<dc:rights>\n"+license+"</dc:rights> ");
		br.write("<dc:publisher>\n"+publisher+"</dc:publisher> ");
		br.write("<dc:identifier id=\"BookId\">urn:uuid:"+uid+"</dc:identifier>\n");
		EPubResource cover = null;
		if(pages.size() > 0 && pages.get(0).resources.size() > 0){
			for(int jj = 0;jj < pages.get(0).resources.size();jj++){
				if(pages.get(0).resources.get(jj).mediaType.indexOf("image") > -1){
					EPubPage.copyFile(dir+"/"+pages.get(0).resources.get(jj).relativePath,dir+"/"+pages.get(0).resources.get(jj).relativePath+"cover");
					cover = new EPubResource("cover-image",pages.get(0).resources.get(jj).relativePath+"cover",pages.get(0).resources.get(jj).mediaType);
					break;
				}
			}
		}
		//cover = null;
		if(cover != null){
			br.write("<meta name=\"cover\" content=\"cover-image\"/>");
		}
		br.write("</metadata>\n");
		br.write("<manifest>\n");
		br.write("<item id=\"ncx\" href=\""+ncxFileName+"\" media-type=\"application/x-dtbncx+xml\" />\n");
		br.write("<item id=\"style\" href=\"style.css\" media-type=\"text/css\" />\n");
		if(cover != null){
			br.write("<item id=\"cover-image\" href=\""+cover.relativePath+"\" media-type=\""+cover.mediaType+"\" />\n");
		
		}
		for(int ii = 0;ii < pages.size();ii++){
			if(pages.get(ii).id.equals("none")){
				pages.get(ii).id = "page"+String.valueOf(Math.random()).replaceAll("[^0-9]","");
			}
			br.write("<item id=\""+pages.get(ii).id+"\" href=\""+pages.get(ii).relativeFilePath+"\" media-type=\""+pages.get(ii).mediaType+"\" />\n");
			for(int jj = 0;jj < pages.get(ii).resources.size();jj++){
				if(pages.get(ii).resources.get(jj).id.equals("none")){
					pages.get(ii).resources.get(jj).id = "res"+String.valueOf(Math.random()).replaceAll("[^0-9]","");;
				}
				br.write("<item id=\""+pages.get(ii).resources.get(jj).id
				+"\" href=\""+pages.get(ii).resources.get(jj).relativePath
				+"\" media-type=\""+pages.get(ii).resources.get(jj).mediaType+"\" />\n");
			}
		}
		
		
		br.write("</manifest>\n");
		if(rtlFlag){
			br.write("<spine toc=\"ncx\" page-progression-direction=\"rtl\">\n");
		}else{
			br.write("<spine toc=\"ncx\" page-progression-direction=\"ltr\">\n");
		}
		for(int ii = 0;ii < pages.size();ii++){
		br.write("<itemref idref=\""+pages.get(ii).id+"\" />\n");
		}
		br.write("</spine>\n");
		br.write("</package>\n");
		br.close();
	}
	
	public void makeNCXFile(File dir) throws IOException{
		
		OutputStreamWriter fw = new OutputStreamWriter(new FileOutputStream(new File(dir.getPath()+"/"+ncxFileName)),"UTF-8");
		BufferedWriter br = new BufferedWriter(fw);
		br.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
		br.write("<ncx xmlns=\"http://www.daisy.org/z3986/2005/ncx/\" version=\"2005-1\">\n");
		br.write("<head>\n");
		br.write("<meta name=\"dtb:uid\" content=\""+uid+"\"/>\n");
		br.write("<meta name=\"dtb:depth\" content=\"1\"/>\n");
		br.write("<meta name=\"dtb:totalPageCount\" content=\"0\"/>\n");
		//br.write("<meta name=\"dtb:totalPageCount\" content=\""+String.valueOf(pages.size())+"\"/>\n");
		br.write("<meta name=\"dtb:maxPageNumber\" content=\"0\"/>\n");
		br.write("</head>\n");
		br.write("<docTitle>\n");
		br.write("<text>\n"+title+"</text>\n");
		br.write("</docTitle>\n");
		br.write("<navMap>\n");
		Hashtable<String,String> usedpage = new Hashtable<String,String>();
		int cou = 1;
		if(indices.size() > 0){
			for(int ii = 0;ii < indices.size();ii++){
				if(usedpage.get(indices.get(ii).getAnchorPoint()) == null){
					br.write("<navPoint id=\"index_"+String.valueOf(ii)+"\" playOrder=\""+String.valueOf(cou)+"\">\n");
					br.write("<navLabel>\n");
					br.write("<text>\n"+indices.get(ii).description+"\n</text>\n");
					br.write("</navLabel>\n");
					
					br.write("<content src=\""+indices.get(ii).getAnchorPoint()+"\"/>\n");
					br.write("</navPoint>\n");
					usedpage.put(indices.get(ii).getAnchorPoint(),String.valueOf(ii));
					cou++;
				}else{
					ErrorMessageBox.append(indices.get(ii).description+" : Target page"+indices.get(ii).target.relativeFilePath+ " was already used.\n");
				}
			}
		}else{
			br.write("<navPoint id=\"start\" playOrder=\"1\">\n");
		    br.write("<navLabel>\n");
		    br.write("<text>start page</text>\n");
		    br.write("</navLabel>\n");
		    br.write("<content src=\""+pages.get(0).relativeFilePath+"\"/>\n");
		    br.write("</navPoint>\n");
			
		}
		br.write("</navMap>\n");
		
		br.write("</ncx>\n");
		br.close();
	}
	
	
	
}

class EPubIndex{
	EPubPage target;
	String description;
	String anchor = null;
	EPubIndex(EPubPage t,String d,String a){
		target = t;
		description = d;
		anchor = a;
	}
	EPubIndex(EPubPage t,String d,int a){
		target = t;
		description = d;
		anchor = "p"+String.valueOf(a);
	}
	EPubIndex(EPubPage t,String d){
		target = t;
		description = d;
	}
	
	public String getAnchorPoint(){
		if(anchor == null){
			return target.relativeFilePath;
		}else{
			return target.relativeFilePath+"#"+anchor;
		}
	}
}
class EPubResource{
	String id = "none";
	String relativePath = "none";
	String mediaType = "none";
	EPubResource(String i,String rel,String mt){
		id = i;
		relativePath = rel;
		mediaType = mt;
	}
}
class EPubPage{
	String hearder = null;
	String title = null;
	String styleSheet = null;
	String styleSection = null;
	String relativeFilePath;
	File sourceFile;
	String mediaType;
	String id = "none";
	String encoding = "UTF-8";
	String langcode = null;
	String contentDirPath;
	ArrayList<EPubResource> resources = new ArrayList<EPubResource>();
	String contentString = "";
	
	EPubPage(String t){
		title = t;
		mediaType = "application/xhtml+xml";
		setStyleSection(getDefaultStyleSection());
	}
	
	public static void copyFile(String s,String d){
		FileChannel srcChannel = null;
		FileChannel destChannel = null;
		try{
			srcChannel = new FileInputStream(s).getChannel();
			destChannel = new FileOutputStream(d).getChannel();
			srcChannel.transferTo(0, srcChannel.size(), destChannel);
		}catch(Exception exx){
			exx.printStackTrace();
			
		} finally {
			try{
				srcChannel.close();
			}catch(Exception exx){
				exx.printStackTrace();
			}
			try{
				destChannel.close();
			}catch(Exception exx){
				exx.printStackTrace();
			}
		}
	}

	public void setStyleSheet(String st){
		styleSheet = st;
	}
	public void setStyleSection(String str){
		styleSection = str;
	}
	public void setContentDirPath(String s){
		contentDirPath = s;
	}
	public void setEncoding(String s){
		encoding = s;
	}
	public void changeHTMLLang(String str){
		langcode = str;
	}
	public void setContentString(String str){
		contentString = str;
	}
	public void createPageFile(String filepath,String resourcedir_relative){
		try{
			File out = new File(filepath);
			relativeFilePath = filepath.replace(contentDirPath,"");//-------------------important!
			relativeFilePath = relativeFilePath.replaceFirst("^/+","");
			OutputStreamWriter fw = new OutputStreamWriter(new FileOutputStream(out),encoding);
			BufferedWriter br = new BufferedWriter(fw);
			br.write(getDefaultDocType());
			if(langcode != null){
				br.write("<html xmlns=\"http://www.w3.org/1999/xhtml\" xml:lang=\""+langcode+"\" lang=\""+langcode+"\">\n");
			}else{
				br.write("<html xmlns=\"http://www.w3.org/1999/xhtml\" xml:lang=\"ja\" lang=\"ja\">\n");
			}
			br.write(getHeaderString());
			br.write("<body>\n");
			br.write(contentString);
			br.write("\n");
			
			br.write("</body>\n</html>\n");
			br.close();
		}catch(Exception exx){
			exx.printStackTrace();
		}
	}
	
	public static String getDefaultDocType(){
		return "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.1//EN\" \"http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd\">\n";
	}	
	public String getHeaderString(){
		StringBuffer sb = new StringBuffer();
		sb.append("<head>\n");
		if(title != null){
			sb.append("<title>\n"+title+"</title>\n");
		}
		if(styleSheet != null){
			sb.append("<link href=\""+styleSheet+"\" type=\"text/css\" rel=\"stylesheet\" />\n");
		}
		if(styleSection != null){
			sb.append(styleSection+"\n");
		}
		sb.append("</head>\n");
		return sb.toString();
	}
	
	public static String getDefaultStyleSection(){
		StringBuffer ret = new StringBuffer();
		ret.append("<meta http-equiv=\"Content-Style-Type\" content=\"text/css\" />\n");
		ret.append("<style type=\"text/css\">\n");
		ret.append("html {padding: 0px;height:100%;margin: 0px;text-align:center;}\n");
		ret.append("body {padding: 0px;height:100%;margin: 0px;text-align:center;}\n");
		ret.append("table {padding: 0px;height:100%;width:100%;margin: 0px;text-align:center;}\n");
		ret.append("p {padding: 0px;margin: auto;text-align:center;}\n");
		//ret.append("img {width:100%;padding:0px;margin:auto;}\n");
		ret.append("img {padding:0px;margin:auto;}\n");
		ret.append("</style>\n");
		return ret.toString();
	}
}

class XHTMLComicPage extends EPubPage{
	ArrayList<File> imgFile = new ArrayList<File>();
	public static Hashtable<String,String> mediaTypes = null;
	XHTMLComicPage(String t,File img){
		super(t);
		title = t;
		mediaType = "application/xhtml+xml";
		setStyleSection(getDefaultStyleSection());
		imgFile.add(img);
	}
	
	public void addNewImage(File f){
		imgFile.add(f);
	}
	public int getPageNum(){
		return imgFile.size();
	}
	public void clearImageList(){
		imgFile.clear();
	}
	public static void setMediaTypes(){
		mediaTypes = new Hashtable<String,String>();
		mediaTypes.put(".gif","image/gif");
		mediaTypes.put(".png","image/png");
		mediaTypes.put(".jpg","image/jpeg");
		mediaTypes.put(".jpeg","image/jpeg");
		
	}
	public String getMediaType(String e){
		if(mediaTypes == null){
			setMediaTypes();
		}
		String ext = e.toLowerCase();
		if(ext.indexOf(".") < 0){
			ext = "."+ext;
		}else if(ext.indexOf(".") > 0){
			ext = EPubCompiler.getExtensionCode(ext);
		}
		return mediaTypes.get(ext);
	}
	public void createPageFile(String filepath,String resourcedir_relative){
		createPageFile(filepath,resourcedir_relative,imgFile);
	}
	public void createPageFile(String filepath,String r,ArrayList<File> img){
		File outf = new File(filepath);
		
		
		
		String rel = r.replace(outf.getParent(),"");
		String resourcedir = outf.getParent()+"/"+rel;
		File res = new File(resourcedir);
		if(!res.exists()){
			res.mkdir();
		}
		ArrayList<String> al = new ArrayList<String>();
		for(int ii = 0;ii < img.size();ii++){
			String imgpath_absolulte = resourcedir+"/"+EPubCompiler.getRandomName(res,"img",EPubCompiler.getExtensionCode(img.get(ii).getName()));
			String imgpath_relative =  imgpath_absolulte.replace(outf.getParent(),"").replaceFirst("/+","");
			//System.out.println(imgpath_relative);
			copyFile(img.get(ii).getPath(),imgpath_absolulte);
			al.add(imgpath_relative);
		}
		createPageFile(filepath,al);
		
	}
	public void addAuthorDataFile(){
		
	}
	public void createPageFile(String filepath,ArrayList<String> imgpath_relative){
		try{
			File out = new File(filepath);
			relativeFilePath = filepath.replace(contentDirPath,"");//-------------------important!
			relativeFilePath = relativeFilePath.replaceFirst("^/+","");
			OutputStreamWriter fw = new OutputStreamWriter(new FileOutputStream(out),encoding);
			BufferedWriter br = new BufferedWriter(fw);
			br.write(getDefaultDocType());
			if(langcode != null){
				br.write("<html xmlns=\"http://www.w3.org/1999/xhtml\" xml:lang=\""+langcode+"\" lang=\""+langcode+"\">\n");
			}else{
				br.write("<html xmlns=\"http://www.w3.org/1999/xhtml\">\n");
			}
			br.write(getHeaderString());
			br.write("<body>\n");
			//br.write("<table cellspacing=\"0px\"  cellpadding=\"0px\" border=\"0px\"><tr><td align = \"center\" valign=\"middle\">");
			for(int ii = 0;ii < imgpath_relative.size();ii++){
				if(ii > 0){
					//br.write("<br style=\"page-break-after:always;\" />\n");
				}
				
				resources.add(new EPubResource("none",imgpath_relative.get(ii),getMediaType(imgpath_relative.get(ii))));
				if(ii > 0){
					br.write("<div id=\"p"+ii+"\"  style=\"page-break-before:always;page-break-inside:avoid;overflow:hidden;\" ><img src=\""+imgpath_relative.get(ii)+"\" alt=\"image\"  style=\"page-break-inside:avoid;\"/></div>\n");
				}else{
					br.write("<div id=\"p"+ii+"\"  style=\"page-break-inside:avoid;\" ><img src=\""+imgpath_relative.get(ii)+"\" alt=\"image\"  style=\"\"/></div>\n");
				}
				//br.write("<h1 style=\"page-break-after:always;\" ><p id=\"p"+ii+"\"><img src=\""+imgpath_relative.get(ii)+"\" alt=\"image\"/></p></h1>\n");
				//br.write("<div style=\"page-break-after:always;page-break-inside:avoid;\" ><p id=\"p"+ii+"\"><img src=\""+imgpath_relative.get(ii)+"\" alt=\"image\"/></p></div>\n");
			}
			//br.write("</td>\n</tr>\n</table>\n");
			br.write("</body>\n</html>\n");
			br.close();
		}catch(Exception exx){
			exx.printStackTrace();
		}
	}
	
}
