import java.awt.*;
import java.awt.event.*;
import java.awt.datatransfer.*;
import java.awt.dnd.*;
import java.io.*;
import javax.swing.*;
import javax.swing.tree.*;
import java.util.*;
import javax.swing.event.*;
import java.awt.image.BufferedImage;
import javax.imageio.*;
public class PPLayerTree extends JTree implements DragSourceListener, DropTargetListener, DragGestureListener,TreeSelectionListener,MouseListener{
	public static final int TREE_INSERTNODE = PPMainPanel.TREE_INSERTNODE;
	public static final int TREE_DELETENODE = PPMainPanel.TREE_DELETENODE;
	public static final int TREE_SELECTIONCHANGED = PPMainPanel.TREE_SELECTIONCHANGED;
	public static final int TREE_APPENDNEWNODE = PPMainPanel.TREE_APPENDNEWNODE;
	public static final int TREE_UPDATENODE = PPMainPanel.TREE_UPDATENODE;
	public static final int EVENTCODE_SETCURRENTDIR = PPMainPanel.EVENTCODE_SETCURRENTDIR;
	public static String SEPARATER_A = ">";
	public static String SEPARATER_B = "<";
	public static final String[] INTERPRETATIONSTRING = {"INSERTNODE","DELETENODE","SELECTIONCHANGED","NEWNODE","UPDATENODE","SETCURRENTDIR"};
	public static final int INSERT_AFTER = 0;
	public static final int INSERT_BEFORE = 1;
	public static final int INSERT_INTO = 2;
	
	int insertPos = INSERT_BEFORE;
	boolean groupDropping = false;
	boolean droppingIntoGroup = false;
	boolean dropToRoot = false;
	boolean dragGestureFlag = false;
	Hashtable<String,Integer> flavorsHash = new Hashtable<String,Integer>();
	ArrayList<TreePath> selectedPath = new ArrayList<TreePath>();
	TreeNode dropTargetNode = null;
	DragGestureRecognizer dragGestureRecognizer;
	DragSource dragSource;
	TreePath[] prevSelection;
	ActionListener parentListener = null;
	Hashtable<String,LabeledNode> nodeHash = new Hashtable<String,LabeledNode>();
	PPLayerTree(){
		super();
		setCellRenderer(new PPLayerTreeCellRenderer(this));
		flavorsHash.put(PPTransferable.LEAF_FLAVOR.getHumanPresentableName(),PPTransferable.CODE_LEAF);
		flavorsHash.put(PPTransferable.GROUP_FLAVOR.getHumanPresentableName(),PPTransferable.CODE_GROUP);
		flavorsHash.put(DataFlavor.javaFileListFlavor.getHumanPresentableName(),PPTransferable.CODE_FILES);
		setRootVisible(false);
		addTreeSelectionListener(this);
		dragSource = new DragSource();
        dragSource.addDragSourceListener(this);
        dragGestureRecognizer = dragSource.createDefaultDragGestureRecognizer(this, DnDConstants.ACTION_MOVE, this);
		new DropTarget(this, this);
		addMouseListener(this);
	}
	
	
	public static void setInterpretationString(Hashtable<String,Integer> s){
		s.put(INTERPRETATIONSTRING[TREE_INSERTNODE],TREE_INSERTNODE);
		s.put(INTERPRETATIONSTRING[TREE_DELETENODE],TREE_DELETENODE);
		s.put(INTERPRETATIONSTRING[TREE_SELECTIONCHANGED],TREE_SELECTIONCHANGED);
		s.put(INTERPRETATIONSTRING[TREE_APPENDNEWNODE],TREE_APPENDNEWNODE);
		s.put(INTERPRETATIONSTRING[TREE_UPDATENODE],TREE_UPDATENODE);
		s.put(INTERPRETATIONSTRING[EVENTCODE_SETCURRENTDIR],EVENTCODE_SETCURRENTDIR);
		
	}
	public void setParentListener(ActionListener ac){
		parentListener = ac;
	}
	public Hashtable<String,LabeledNode> getNodeHash(){
		return nodeHash;
		
	}
	
	public void copyFrom(PPLayerTree pt){
		DefaultTreeModel mod = (DefaultTreeModel)pt.getModel();
		LabeledNode rootnode = (LabeledNode)mod.getRoot();
		DefaultTreeModel mod_this = (DefaultTreeModel)this.getModel();
		LabeledNode root_this = (LabeledNode)mod_this.getRoot();
		root_this.copyFrom(rootnode,true);
		nodeHash.clear();
		nodeHash.put("ROOT",root_this);
		
		ArrayList<LabeledNode> al = getAllChildren(root_this);
		for(int ii = 0;ii < al.size();ii++){
			LabeledNode lb = al.get(ii);
			nodeHash.put(lb.getId(),lb);
		}
	}
	
	public ArrayList<LabeledNode> getAllChildren(LabeledNode l){
		ArrayList<LabeledNode> ret =  new  ArrayList<LabeledNode>();
		Enumeration enu = l.children();
		while(enu.hasMoreElements()){
			LabeledNode dt = (LabeledNode)enu.nextElement();
			ret.add(dt);
			if(dt.getChildCount() != 0){
				ret.addAll( getAllChildren(dt));
			}
		}
		return ret;
	}
	
	public void mouseClicked(MouseEvent e){
		
	}
	public void mouseReleased(MouseEvent e){
		dropTargetNode = null;
	}
	public void mousePressed(MouseEvent e){
		TreePath path = getPathForLocation_Y(e.getX(),e.getY());
		if(isRowSelected(getClosestRowForLocation(e.getX(),e.getY()))){
		}else{
			setSelectionPath(path);
		}
	}
	public void mouseEntered(MouseEvent e){
		
	}
	public void mouseExited(MouseEvent e){
		dropTargetNode = null;
	}
	
	public void mouseDragged(MouseEvent e){
		
	}
	
	public void valueChanged(TreeSelectionEvent e){
		TreePath[] ta = this.getSelectionPaths();
		if(ta == null){
			prevSelection = null;
			return;
		}
		if(ta.length == 1 && prevSelection != null){
			boolean flag = false;
			for(int ii = 0;ii < prevSelection.length;ii++){
				if(ta[0] == prevSelection[ii]){
					flag = true;
					break;
				}
			}
			if(flag){
				addSelectionPaths(prevSelection);
			}
		}
		prevSelection = ta;
		ta = this.getSelectionPaths();
	
		StringBuffer actionstringbuffer = new StringBuffer();
		actionstringbuffer.append("!");
		actionstringbuffer.append(SEPARATER_A+"ACTION="+INTERPRETATIONSTRING[TREE_SELECTIONCHANGED]+SEPARATER_B+"ID=");
		for(int ii = 0;ii < ta.length;ii++){
			LabeledNode tnode = (LabeledNode)ta[ii].getLastPathComponent();
			actionstringbuffer.append(tnode.getId()+":::");
		}
		if(parentListener != null){
			parentListener.actionPerformed(new ActionEvent(this,ActionEvent.ACTION_PERFORMED,actionstringbuffer.toString()));
		}
		
	}
	
	public ArrayList<LabeledNode> getSelectedNodes(){
		ArrayList<LabeledNode> ret = new ArrayList<LabeledNode>();
		TreePath[] ta = this.getSelectionPaths();
		if(ta == null){
			return ret;
		}
		for(int ii = 0;ii < ta.length;ii++){
			LabeledNode no = (LabeledNode)ta[ii].getLastPathComponent();
			ret.add(no);
		}
		return ret;
	}
	public void dragGestureRecognized(DragGestureEvent e){
		Point p = e.getDragOrigin();
		dragGestureFlag = true;
		ArrayList<TreePath> list = new ArrayList<TreePath>();
		TreeModel tm = this.getModel();
		Object rootnode = tm.getRoot();
		TreePath[] ta = this.getSelectionPaths();
		boolean groupflag = false;
		selectedPath.clear();
		if(ta == null){
			return;
		}
		for(int ii = 0;ii < ta.length;ii++){
			list.add(ta[ii]);
			selectedPath.add(ta[ii]);
			TreeNode no = (TreeNode)ta[ii].getLastPathComponent();
			groupflag = (groupflag || isGroupNode(ta[ii]));
			
		}
		Transferable tra;
		try{
			if(groupflag){
				 tra = new PPTransferable(list,PPTransferable.GROUP_FLAVOR);
			}else{
				 tra = new PPTransferable(list,PPTransferable.LEAF_FLAVOR);
			}
		}catch(Exception exx){
			exx.printStackTrace();
			selectedPath.clear();
			return;
		}
		new DragSource().startDrag(e,Cursor.getDefaultCursor(),tra,this);
	}
	
	public void dragDropEnd(DragSourceDropEvent e){
		
		dropToRoot = false;
	}
	public void dragEnter(DragSourceDragEvent e){
		e.getDragSourceContext().setCursor(DragSource.DefaultMoveDrop);
	}
	public void dragExit(DragSourceEvent e){
		e.getDragSourceContext().setCursor(DragSource.DefaultMoveNoDrop);
	}
	public void dragOver(DragSourceDragEvent e){
	}
	
	public void dropActionChanged(DragSourceDragEvent e){
	}
	
	
	public void dropActionChanged(DropTargetDragEvent e){
	}
	public void dragEnter(DropTargetDragEvent e){
	}
	public void dragExit(DropTargetEvent e){
	}
	public void dragOver(DropTargetDragEvent e){
		DataFlavor[] f = e.getCurrentDataFlavors();
		
		Point p = e.getLocation();
		TreePath path = getPathForLocation_Y(p.x,p.y);
		if(path == null){
			dropToRoot = true;
			e.acceptDrag(e.getDropAction());
			repaint();
			dropTargetNode = null;
			return;
			//rejectDrag(e);
			//return;
		}
		dropToRoot = false;
		dropTargetNode = (TreeNode)path.getLastPathComponent();
		for(int ii = 0;ii < selectedPath.size();ii++){
			if(selectedPath.get(ii).isDescendant(path)){
				rejectDrag(e);
				return;
			
			}
			if(path.equals(selectedPath.get(ii))){
				rejectDrag(e);
				return;
			}
		}
		
		
		boolean groupflag = isGroupNode(path);
		
		droppingIntoGroup = groupflag;
		
		if(flavorsHash.get(f[0].getHumanPresentableName()) == null){
			rejectDrag(e);
			return;
		}
		
		
		switch(flavorsHash.get(f[0].getHumanPresentableName())){
			case PPTransferable.CODE_LEAF:
				groupDropping = false;
				
			break;
			case PPTransferable.CODE_GROUP:
				groupDropping = true;
				
			break;
		}
		
		Rectangle droprect = getPathBounds(path);
		if(!groupflag){
			if(p.y > droprect.y+droprect.height/2){
				insertPos = INSERT_AFTER;
			}else{
				insertPos = INSERT_BEFORE;
			}
		}else{
			if(p.y < droprect.y+droprect.height/4*3 && p.y > droprect.y+droprect.height/4){
				insertPos = INSERT_INTO;
			}else if(p.y > droprect.y+droprect.height/2){
				insertPos = INSERT_AFTER;
			}else{
				insertPos = INSERT_BEFORE;
			}
			
		}
		
		
		
		e.acceptDrag(e.getDropAction());
		repaint();
	}
	public void dropComplete(DropTargetDropEvent e,boolean flag){
		//selectedPath.clear();
		dragGestureFlag = false;
		e.dropComplete(flag);
		dropTargetNode = null;
		dropToRoot = false;
		repaint();
	}
	public void rejectDrag(DropTargetDragEvent e){
		//selectedPath.clear();
		e.rejectDrag();
		dragGestureFlag = false;
		dropTargetNode = null;
		repaint();
	}
	public boolean isGroupNode(TreePath path){
		if(path == null)return false;
		DefaultTreeModel model = (DefaultTreeModel)this.getModel();
		
		LabeledNode no = (LabeledNode)path.getLastPathComponent();
		if(!no.isLeafLabel()){
			return true;
		}
		return false;
	}
	public TreeNode getNearestGroupNode(TreePath path){
		DefaultTreeModel model = (DefaultTreeModel)this.getModel();
		TreeNode no = (TreeNode)path.getLastPathComponent();
		if(!model.isLeaf(no)){
			return no;
		}else{
			return no.getParent();
		}
	}
	public LabeledNode getNearestGroup(TreePath path){

		if(path == null){
			return null;
		}
		LabeledNode no = (LabeledNode)path.getLastPathComponent();
		if(no.isGroupNode()){
			return no;
		}else{
			return (LabeledNode)no.getParent();
		}
	}
	public LabeledNode getNearestGroup(){
		TreePath[] ta = this.getSelectionPaths();
		if(ta == null){
			return null;
		}
		return getNearestGroup(ta[0]);
	}
	public String insertAllFiles(ArrayList<File> al,String parentid){
		return insertAllFiles(al,parentid,-1);
	}
	
	
	@SuppressWarnings("unchecked")public ArrayList<File> sortAllFiles_L(ArrayList<File> fileList){
		ArrayList<File> fal = new ArrayList<File>();
		ArrayList<Long> used = new ArrayList<Long>();
		Hashtable<Long,File> table = new Hashtable<Long,File>();
		
		String[] biggest = "0,0,0".split(",");
		for(int jj = 0;jj < fileList.size();jj++){
			File f = (File)fileList.get(jj);
			String[] n = f.getName().split("[^0-9]+");
			int cou = 0;
			for(int kk = n.length-1;kk >= 0;kk--){
				if(n[kk].length() > 0){
					if(Long.parseLong(n[kk]) > Long.parseLong(biggest[cou])){
						biggest[cou] = n[kk];
					}
					cou++;
					if(cou >= biggest.length){
						break;
					}
				}
			}
		}
		for(int jj = 0;jj < biggest.length;jj++){
			biggest[jj] = biggest[jj].replaceAll(".","0");
		}
		
		//9223372036854775808
		for(int jj = 0;jj < fileList.size();jj++){
			File f = (File)fileList.get(jj);
			String[] n = f.getName().split("[^0-9]+");
			int cou = 0;
			StringBuffer mapped = new StringBuffer();
			//System.out.println(f.getName());
			for(int kk = n.length-1;kk >= 0;kk--){
				if(n[kk].length() > 0){
					if(Long.parseLong(n[kk]) > Long.parseLong(biggest[cou])){
						String cm = biggest[cou]+n[kk];
						mapped.insert(0,cm.substring(cm.length()-biggest[cou].length(),cm.length()));
					}
					cou++;
					if(cou >= biggest.length){
						break;
					}
				}
			}
			String tcp = mapped.toString()+"0";
			if(tcp.length() > 17){
				tcp = tcp.substring(tcp.length()-17,tcp.length());
				
			}
			long lp = Long.parseLong(tcp);
			while(table.get(lp) != null){
				lp++;
			}
			
			//System.out.println(lp);
			table.put(lp,f);
			used.add(lp);
		}
		
		Object[] sorted = used.toArray();
		Arrays.sort(sorted, new LongComparator());
		for(int jj = 0;jj < sorted.length;jj++){
			long c = (Long)sorted[jj];
			fal.add(table.get(c));
		}
		return fal;
	}
	
	@SuppressWarnings("unchecked")public ArrayList<File> sortAllFiles(ArrayList<File> fileList){
		ArrayList<File> fal = new ArrayList<File>();
		
		Object[] sorted = fileList.toArray();
		Arrays.sort(sorted, new FileComparator());
		for(int jj = 0;jj < sorted.length;jj++){
			fal.add((File)sorted[jj]);
		}
		return fal;
	}
	public String insertAllFiles(ArrayList<File> al,String parentid,int pos){
		StringBuffer ret = new StringBuffer(); 
		ret.append(SEPARATER_A+"ACTION="+INTERPRETATIONSTRING[TREE_APPENDNEWNODE]+SEPARATER_B);
		ret.append("PARENT="+parentid+SEPARATER_B);
		if(pos > -1){
			ret.append("POS="+pos+SEPARATER_B);
			
		}
		ret.append("DATA=");
		
		ArrayList<File> dirlist = new ArrayList<File>();
		StringBuffer idsb = new StringBuffer();
		idsb.append("ID=");
		al = sortAllFiles(al);
		for(int ii = 0;ii < al.size();ii++){
			File f = al.get(al.size()-1-ii);
			if(f.getName().indexOf(".") > 0){
				if(f.isFile()){
					ret.append("FILEPATH=");
					ret.append(f.getPath());
					ret.append(":::");
					idsb.append(LabeledNode.createNewId()+":::");
				}else if(f.isDirectory()){
					dirlist.add(f);
				}
			}else if(f.isDirectory()){
				dirlist.add(f);
			}
		}
		
		StringBuffer dirbuffer = new StringBuffer();
		for(int ii = 0;ii < dirlist.size();ii++){
				File f = dirlist.get(ii);
				//System.out.println(f.getPath());
				ret.append("GROUP=");
				ret.append(f.getName());
				ret.append(":::");
				String id = String.valueOf(LabeledNode.createNewId());
				idsb.append(id+":::");
				File[] fl = f.listFiles();
				if(fl.length > 0){
					ArrayList<File> fal = new ArrayList<File>();
					for(int jj = 0;jj < fl.length;jj++){
						fal.add(fl[jj]);
					}
					dirbuffer.append(insertAllFiles(fal,id));
				}
		}
		
		//System.out.println("OO");
		ret.append(SEPARATER_B);
		ret.append(idsb);
		ret.append(SEPARATER_B);
		ret.append(dirbuffer);
		ret.append(SEPARATER_B);
		
		
		/*//-----position
		ArrayList<LabeledNode> sel = getSelectedNodes();
		if(sel.size() > 0){
			LabeledNode targetnode = sel.get(sel.size()-1);
			if(targetnode.getParent() != null){
				String pid = "";
				LabeledNode pln = (LabeledNode)targetnode.getParent();
				pid = pln.getId();
				if(pid.equals(parentid)){
					ret.append("POS="+String.valueOf(pln.getIndex(targetnode)+1));
				}
			}
		}*/
		
		
		dropToRoot = false;
		//System.out.println(ret);
		return ret.toString();
	}
	
	public String copyAllNodes(ArrayList<LabeledNode> al,String parentid){
		StringBuffer ret = new StringBuffer(); 
		ret.append(SEPARATER_A+"ACTION="+INTERPRETATIONSTRING[TREE_APPENDNEWNODE]+SEPARATER_B);
		ret.append("DATA=");
		
		ArrayList<LabeledNode> dirlist = new ArrayList<LabeledNode>();
		StringBuffer idsb = new StringBuffer();
		idsb.append("ID=");
		for(int ii = 0;ii < al.size();ii++){
			LabeledNode f = al.get(al.size()-1-ii);
			if(!f.isGroupNode()){
				
				ret.append(f.getCreationCode());
				ret.append(":::");
				idsb.append("#"+LabeledNode.createNewId()+":::");
			}else{
				dirlist.add(f);
			}
		
		}
		StringBuffer dirbuffer = new StringBuffer();
		for(int ii = 0;ii < dirlist.size();ii++){
				LabeledNode f = dirlist.get(ii);
				ret.append(f.getCreationCode());
				ret.append(":::");
				String id = String.valueOf(LabeledNode.createNewId());
				idsb.append("#"+id+":::");
				if(f.isGroupNode()){
					ArrayList<LabeledNode> fal = new ArrayList<LabeledNode>();
					int cou = f.getChildCount();
					if(cou > 0){
						for(int jj = 0;jj < cou;jj++){
							fal.add((LabeledNode)f.getChildAt(jj));
						}
						dirbuffer.append(copyAllNodes(fal,"#"+id));
					}
				}
		}
		
		ret.append(SEPARATER_B);
		ret.append(idsb);
		
		if(parentid.length() > 0){
			ret.append(SEPARATER_B);
			ret.append("PARENT="+parentid+SEPARATER_B);
		}
		ret.append(SEPARATER_B);
		ret.append(dirbuffer);
		
		return ret.toString();
	}
	
	
	@SuppressWarnings("unchecked")public void drop(DropTargetDropEvent e){
		ArrayList<TreePath> droppedList = null;
		
		StringBuffer actionstringbuffer = new StringBuffer();
		Transferable transfer = e.getTransferable();
		try{
			if (transfer.isDataFlavorSupported( DataFlavor.javaFileListFlavor ) ) {
				e.acceptDrop(DnDConstants.ACTION_COPY_OR_MOVE);
				Point tp = e.getLocation();
				
				TreePath ppath = getPathForLocation_Y(tp.x,tp.y);
				
				java.util.List fileList =(java.util.List)( transfer.getTransferData( DataFlavor.javaFileListFlavor ) );
				
				e.dropComplete(true);
				
				ArrayList<File> pal = new ArrayList<File>();
				for(int jj = 0;jj < fileList.size();jj++){
					pal.add((File)(fileList.get(jj)));
				}
				if(pal.size() > 0){
					parentListener.actionPerformed(new ActionEvent(this,ActionEvent.ACTION_PERFORMED,"!"+SEPARATER_A+"ACTION=SETCURRENTDIR"+SEPARATER_B+"DIR="+pal.get(0).getParent()+SEPARATER_B));
				}else{
					return;
				}
				
				
				ArrayList<File> fal = sortAllFiles(pal);
				
				String parentid = "ROOT";
				if(ppath != null){ 
					LabeledNode ln = (LabeledNode)ppath.getLastPathComponent();
					if(ln.type == LabeledNode.TYPE_GROUP){
						parentid = ln.id;
					}else{
						parentid = ((LabeledNode)ln.getParent()).id;
					}
				}
				int pos = -1;
				if(ppath != null){
					LabeledNode ln = (LabeledNode)ppath.getLastPathComponent();
					if(ln.type == LabeledNode.TYPE_GROUP){
						pos=ln.getChildCount();
					}else{
						
						LabeledNode parentnode = ((LabeledNode)ln.getParent());
						pos = parentnode.getIndex(ln)+((insertPos==INSERT_BEFORE)?(0):(1));
						
						//actionstringbuffer.append("POS="+String.valueOf(parentnode.getIndex(ln)+((insertPos==INSERT_BEFORE)?(0):(1)))+SEPARATER_B);
					}
				}
				actionstringbuffer.append(insertAllFiles(fal,parentid,pos)+SEPARATER_B);
				
				if(parentListener != null){
					//System.out.println(actionstringbuffer);
					parentListener.actionPerformed(new ActionEvent(this,ActionEvent.ACTION_PERFORMED,actionstringbuffer.toString()));
				}
				return;
			}
		}catch(Exception exx){
			
			ErrorMessageBox.append(exx.getMessage());
			dropComplete(e,false);
			return ;
			
		}
		
		
		
		boolean groupdropped = false;
		try{
			
			Transferable tf = e.getTransferable();
			if(tf.isDataFlavorSupported(PPTransferable.LEAF_FLAVOR)){
				droppedList = (ArrayList<TreePath>)tf.getTransferData(PPTransferable.LEAF_FLAVOR);
			}else if(tf.isDataFlavorSupported(PPTransferable.GROUP_FLAVOR)){
				droppedList = (ArrayList<TreePath>)tf.getTransferData(PPTransferable.GROUP_FLAVOR);
				groupdropped = true;
			}
		}catch(Exception exx){
			exx.printStackTrace();
			dropComplete(e,false);
			return;
		}
		
		Point p = e.getLocation();
		LabeledNode targetnode = null;
		TreePath path = getPathForLocation_Y(p.x,p.y);
		
		if(path == null){ 
			//dropComplete(e,false);
			//return;
			targetnode= ((LabeledNode)this.getModel().getRoot());
			insertPos = INSERT_INTO;
			dropToRoot = true;
		}else{
			for(int ii = 0;ii < selectedPath.size();ii++){
				if(path.equals(selectedPath.get(ii))){
					dropComplete(e,false);
					return;
				}
			}
			targetnode = (LabeledNode)path.getLastPathComponent();
			dropToRoot = false;
		}
		boolean groupflag = isGroupNode(path);
		
		
		DefaultTreeModel model = (DefaultTreeModel)this.getModel();
		DefaultMutableTreeNode parentnode = (DefaultMutableTreeNode)targetnode.getParent();
		for(int ii = 0;ii < selectedPath.size();ii++){
			LabeledNode rn = (LabeledNode)selectedPath.get(ii).getLastPathComponent();
			model.removeNodeFromParent(rn);
			//System.out.println(ii);
		}
		if(groupdropped){
			ArrayList<TreePath> fragment = new ArrayList<TreePath>();
			ArrayList<TreePath> child = new ArrayList<TreePath>();
			for(int ii = 0;ii < droppedList.size();ii++){
				TreePath cp = droppedList.get(ii);
				if(!isGroupNode(cp)){
					boolean cflag = false;
					for(int jj = 0;jj < droppedList.size();jj++){
						if(cp != droppedList.get(jj) && droppedList.get(jj).isDescendant(cp)){
							cflag = true;
							break;
						}
						
					}
					if(!cflag){
						fragment.add(cp);
					}else{
						child.add(cp);
					}
				}else{
				}
			}
			for(int ii = 0;ii < fragment.size();ii++){
				droppedList.remove(fragment.get(ii));
			}
			for(int ii = 0;ii < child.size();ii++){
				droppedList.remove(child.get(ii));
			}
			LabeledNode lastnode = (LabeledNode)droppedList.get(droppedList.size()-1).getLastPathComponent();
			
			for(int ii = 0;ii < fragment.size();ii++){
				LabeledNode ln = (LabeledNode)fragment.get(ii).getLastPathComponent();
				actionstringbuffer.append(SEPARATER_A+"ACTION="+INTERPRETATIONSTRING[TREE_INSERTNODE]+SEPARATER_B);
				actionstringbuffer.append("ID="+ln.getId()+SEPARATER_B);
				actionstringbuffer.append("PARENT="+targetnode.getId()+SEPARATER_B);
				actionstringbuffer.append("POS="+String.valueOf(targetnode.getChildCount()));
				//lastnode.insert(ln,lastnode.getChildCount());
			}
		}
		if(!dropToRoot && (!groupflag || insertPos != INSERT_INTO)){
		
			String parentid;
			LabeledNode pln = (LabeledNode)parentnode;
			parentid = pln.getId();
			
			for(int ii = 0;ii < droppedList.size();ii++){
				LabeledNode ln = (LabeledNode)droppedList.get(ii).getLastPathComponent();
				
				actionstringbuffer.append(SEPARATER_A+"ACTION="+INTERPRETATIONSTRING[TREE_INSERTNODE]+SEPARATER_B);
				actionstringbuffer.append("ID="+ln.getId()+SEPARATER_B);
				actionstringbuffer.append("PARENT="+parentid+SEPARATER_B);
				actionstringbuffer.append("POS="+String.valueOf(parentnode.getIndex(targetnode)+((insertPos==INSERT_BEFORE)?(0):(1))));
			}
		}else{
			for(int ii = 0;ii < droppedList.size();ii++){
				LabeledNode ln = (LabeledNode)droppedList.get(ii).getLastPathComponent();
				actionstringbuffer.append(SEPARATER_A+"ACTION="+INTERPRETATIONSTRING[TREE_INSERTNODE]+SEPARATER_B);
				actionstringbuffer.append("ID="+ln.getId()+SEPARATER_B);
				actionstringbuffer.append("PARENT="+targetnode.getId()+SEPARATER_B);
				actionstringbuffer.append("POS="+String.valueOf(targetnode.getChildCount()));
				//model.insertNodeInto(ln,targetnode,targetnode.getChildCount());
			}
		}
		dropToRoot = false;
		dropComplete(e,true);
		if(parentListener != null){
			parentListener.actionPerformed(new ActionEvent(this,ActionEvent.ACTION_PERFORMED,actionstringbuffer.toString()));
		}
	}
	public void dropActionChanged(DropTargetEvent e){}
	
	public TreePath getPathForLocation_Y(int x,int y){
		
		double wid = this.getWidth();
		TreePath ppath = getPathForLocation(x,y);
		if(ppath == null){
			for(double dd = 0.1;dd < 0.9;dd+=0.1){
				ppath = getPathForLocation((int)(wid*dd),y);
				if(ppath != null){break;};
			}
		}
		return ppath;
	}
	
	public void paintComponent(Graphics g){
		super.paintComponent(g);
		if(getRowCount() > 0 && dropToRoot){
			g.setColor(Color.BLACK);
			Rectangle r = getRowBounds(getRowCount()-1);
			g.drawLine(2,(int)(r.getY()+r.getHeight()+1),(int)(r.getX()+r.getWidth()),(int)(r.getY()+r.getHeight()+1));
		}
	}

	
}
class LongComparator implements java.util.Comparator{
	public int compare(Object o1, Object o2){
		return (int)(((Long)o1) - ((Long)o2));
	}
}

class FileComparator implements java.util.Comparator{
	public int compare(Object o1, Object o2){
		File f1 = (File)o1;
		File f2 = (File)o2;
		
		return f1.getPath().compareTo(f2.getPath());
	}
}
class PPLayerTreeCellRenderer extends DefaultTreeCellRenderer {
	public static final int ICON_BOOKMARK = 0;
	public static final int ICON_DIRECTORY = 1;
	public static final int ICON_DIRECTORYOPEN = 2;
	public static final int ICON_IMAGE = 3;
	public static final int ICON_TEXT = 4;
	public static final String ICONFILE[] = {"./icons/bookmark.png",
												"./icons/directory.png",
												"./icons/directory_open.png",
												"./icons/image.png",
												"./icons/text.png"
												};
	
	static BufferedImage icons[] = null;
	boolean dropTargetFlag = false;
	PPLayerTree parent = null;
	int icon_code = 0;
	Object target = null;
	public PPLayerTreeCellRenderer(PPLayerTree p) {
		super();
		parent = p;
		ImageIcon ico = new ImageIcon(new BufferedImage(1,1,BufferedImage.TYPE_4BYTE_ABGR));
		setClosedIcon(ico);
		setLeafIcon(ico);
		setOpenIcon(ico);
		try{
				if (icons == null){
				icons = new BufferedImage[ICONFILE.length];
				for(int ii = 0;ii < ICONFILE.length;ii++){
					icons[ii] = ImageIO.read(new File(ICONFILE[ii]));
					
				}
			}
		}catch(Exception exx){
			exx.printStackTrace();
		}
	}
	public Component getTreeCellRendererComponent(JTree tree,Object value,boolean isSelected,boolean isExpanded,boolean isLeaf,int row,boolean hasFocus) {
		String dummy = "";
		target = value;
		if(value.getClass() == LabeledNode.class){
			LabeledNode ln = (LabeledNode)value;
			if(ln.type == LabeledNode.TYPE_GROUP){
				 if(isExpanded){
					dummy = ICON_DIRECTORYOPEN+" "+value.toString();
					icon_code = ICON_DIRECTORYOPEN;
				}else{
					dummy = ICON_DIRECTORY+" "+value.toString();
					icon_code = ICON_DIRECTORY;
				}
				
			}else if(ln.type == LabeledNode.TYPE_FILE){
				dummy = ICON_IMAGE+" "+value.toString();
				icon_code = ICON_IMAGE;
			}else if(ln.type == LabeledNode.TYPE_BOOKMARK){
				dummy = ICON_BOOKMARK+" "+value.toString();
				icon_code = ICON_BOOKMARK;
			}else{
				dummy = ICON_TEXT+" "+value.toString();
				icon_code = ICON_TEXT;
			}
			
		}
		return super.getTreeCellRendererComponent(tree,dummy, isSelected, isExpanded, isLeaf, row, hasFocus);
	}
	public void paintComponent(Graphics g) {
		super.paintComponent(g);
		if(target != null && parent.dropTargetNode != null){
			dropTargetFlag = (target == parent.dropTargetNode);
		}else{
			dropTargetFlag = false;
		}
		if(dropTargetFlag) {
			g.setColor(Color.BLACK);
			
			if(parent.insertPos == parent.INSERT_AFTER){
				g.drawLine(0,getSize().height-1,getSize().width,getSize().height-1);
			}else if(parent.insertPos == parent.INSERT_BEFORE){
				g.drawLine(0,0,getSize().width,0);
			}else{
			g.drawRect(0,0,getSize().width-1,getSize().height-1);
			}
		}
		g.drawImage(icons[icon_code],0,0,this);
	}
}//CellRenderer
class PPTransferable implements Transferable{
	public static final int CODE_LEAF = 0;
	public static final int CODE_GROUP = 1;
	public static final int CODE_FILES = 2;
	public static final DataFlavor LEAF_FLAVOR = new DataFlavor(String.class,"PPLAYER_LEAF");
	public static final DataFlavor GROUP_FLAVOR = new DataFlavor(String.class,"PPLAYER_GROUP");
	public static final DataFlavor[] LEAF_SUPPORT = {LEAF_FLAVOR};
	public static final DataFlavor[] GROUP_SUPPORT = {GROUP_FLAVOR};
	  
	Object object;
	DataFlavor myFlavor;
	DataFlavor[] supportedFlavors;
	
	PPTransferable(Object o,DataFlavor mf) throws UnsupportedFlavorException{
		this(o,mf,null);
	}
	PPTransferable(Object o,DataFlavor mf,DataFlavor[] mfs) throws UnsupportedFlavorException{
		object = o;
		myFlavor = mf;
		if(mfs == null){
			if(mf == LEAF_FLAVOR){
				supportedFlavors = LEAF_SUPPORT;
			}else if(mf == GROUP_FLAVOR){
				supportedFlavors = GROUP_SUPPORT;
			}else{
				throw new UnsupportedFlavorException(mf);
			}
		}else{
			supportedFlavors = mfs;
		}
	}
	public Object getTransferData(DataFlavor df) throws UnsupportedFlavorException{
		if(isDataFlavorSupported(df)){
			return object;
		}else{
			throw new UnsupportedFlavorException(df);
		}
	}
	public boolean isDataFlavorSupported(DataFlavor df){
		return (df.getHumanPresentableName().equals(myFlavor.getHumanPresentableName()));
	}
	public DataFlavor[] getTransferDataFlavors(){
		return supportedFlavors;
	}
}



